"""
MS Teams / Email Logger
"""
__title__ = 'aws-teams-logger'
__description__ = 'Python MS Teams and Email logger for AWS environments'
__url__ = 'https://github.com/rnag/aws-teams-logger'
__version__ = '0.5.2'
__author__ = 'Ritvik Nag'
__author_email__ = 'rv.kvetch@gmail.com'
__license__ = 'MIT'
__copyright__ = 'Copyright 2021 Ritvik Nag'
